# Vector Graphics Library
# vgllib.py
import pygame
import uuid
import time
import threading
import math
import copy

class Aux():
    def gettime():
        return round(time.time(), 1)
    def getprogress(task):
        return (Aux.gettime() - task["start"] / task["duration"])
    def tuple_scale(tup, mul):
        return (tup[0]*mul, tup[1]*mul)
class Element(object):
    poscale: tuple = (0, 0)
    is_hide = False
    is_template = True
    attached = None
    tasks = dict()
    def __init__(self, name="Element"):
        self.name = name
    def attach(self, clone_name, frame_object, poscale):
        if clone_name == "":
            clone_name = str(uuid.uuid4())
        clone = copy.deepcopy(self)
        clone.poscale = poscale
        clone.is_template = False
        frame_object.elements[clone_name] = clone
    def teleport(self, delta: tuple):
        if not self.is_template:
            self.poscale[0] += delta[0]
            self.poscale[1] += delta[1]
    def add_task(self, group, start_time, duration, **kwargs):
        if group not in self.tasks():
            self.tasks[group] = list() 
        self.tasks[group].append({"start": start_time, "duration":duration, "para":kwargs})
    def update():
        pass

class Frame(object):
    elements = dict()
    is_hide = False
    is_template = True
    poscale: tuple = (0, 0) # 左上角, 对于每个实例的位置矢量
    def __init__(self, name="Frame", scale: tuple=(1, 1)):
        self.name = name
        self.scale = scale
    def attach(self, clone_name, window_object, poscale):
        if clone_name == "":
            clone_name = str(uuid.uuid4())
        clone = copy.deepcopy(self)
        clone.poscale = poscale
        clone.is_template = False
        window_object.frames[clone_name] = clone
    def render():
        pass

class Window(object):
    frames = list()
    def __init__(self, title="Vector Graphic Layer Window", size: tuple=(1024,768)):
        self.title = title
        self.size = size
        self.running = 1
        pygame.init()
        self.window = pygame.display.set_mode(size)
        pygame.display.set_caption(title)
    def set_title(self, title):
        self.title = title
        pygame.display.set_caption(title)
    def start(self):
        self.thr = threading.Thread(target=self.main_loop, name="main_loop")
        self.thr.start()
    def kill(self):
        self.running = 0
    def main_loop(self):
        while self.running:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.running = 0
            self.window.fill((0, 0, 0))
            pygame.display.flip()
            pygame.time.delay(10)
        pygame.quit()

if __name__ == "__main__":
    pass